
#include "x.h"
#include "xt.h"
#include "globals.h"
#include "defs.h"
#include "externs.h"
#include <stdio.h>
#ifdef XPM
#include <X11/xpm.h>
#endif

Widget reflectColorLabel, materialColorLabel,
       reflectAmbientLabel, reflectDiffuseLabel, reflectTransmissionLabel,
       reflectSpecularLabel, reflectSpecularConcLabel;

#ifdef XPM
void errorMessageXPM(ErrorStatus, tag)
int ErrorStatus;
char *tag;
{
    char *error = NULL;
    char *warning = NULL;

    switch (ErrorStatus) {
    case XpmSuccess:
        return;
    case XpmColorError:
        warning = "Could not parse or alloc requested color";
        break;
    case XpmOpenFailed:
        error = "Cannot open file";
        break;
    case XpmFileInvalid:
        error = "Invalid XPM file";
        break;
    case XpmNoMemory:
        error = "Not enough memory";
        break;
    case XpmColorFailed:
        error = "Failed to parse or alloc some color";
        break;
    }

    if (warning)
        printf("%s Xpm Warning: %s.\n", tag, warning);

    if (error) {
        printf("%s Xpm Error: %s.\n", tag, error);
/*      exit( -1 ); */
    }
}
#endif


void
create_gui( depth, vendor )
int depth;
char *vendor;
{
    int status, i;
    float topOfThumb;
    XtArgVal *l_topOfThumb;
    String string = (String) malloc(30);
    Dimension width, height, widthScr;

    extern PEXReflectionAttributes refl_props;
    extern PEXColor colorMaterial;

    extern Widget toplevel;
    Widget quit, form,
           reflectForm,
           reflectLabel, reflectColor,
           reflectAmbient, reflectDiffuse, reflectTransmission,
           reflectSpecular, reflectSpecularConc,
           reflectAmbientScroll, reflectDiffuseScroll,
           reflectTransmissionScroll,
           reflectSpecularScroll, reflectSpecularConcScroll,

           materialColor, predefMaterialButton, predefMaterialMenu;

#ifdef XPM
    Pixmap backPixmap, pix;
    XpmAttributes attributes;

    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) ) {
        attributes.valuemask = XpmReturnPixels;
        status = XpmReadFileToPixmap( XtDisplay(toplevel),
            DefaultRootWindow(XtDisplay(toplevel)),
            "pixmaps/background.xpm", &backPixmap, &pix, &attributes );
        errorMessageXPM( status, "Background" );
    }
#endif

    /* Create the widgets. */
    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
        NULL );

    canvas = XtVaCreateManagedWidget( "canvas", formWidgetClass, form,
        XtNwidth, 210,
        XtNheight, 210,
        NULL );
    XtAddEventHandler( canvas, ExposureMask, FALSE, expose, NULL );

    /* The reflectance properties panel. */
    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    reflectForm = XtVaCreateManagedWidget( "reflectForm", formWidgetClass,
        form,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    reflectForm = XtVaCreateManagedWidget( "reflectForm", formWidgetClass,
        form,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
        NULL );

    reflectLabel = XtVaCreateManagedWidget( "Label", labelWidgetClass,
        reflectForm,
        XtNlabel, "Reflection Attributes:",
        XtNborderWidth, 0,
        NULL );

    reflectAmbient = XtVaCreateManagedWidget( "Ambient", labelWidgetClass,
        reflectForm,
        XtNlabel, "Amb Coeff:",
        XtNwidth, 122,
        XtNfromVert, reflectLabel,
        NULL );
    XtVaGetValues( reflectAmbient, XtNheight, &height, NULL );

    topOfThumb = refl_props.ambient;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    reflectAmbientScroll = XtVaCreateManagedWidget( "AmbientScroll",
        scrollbarWidgetClass,
        reflectForm,
        XtNfromVert, reflectLabel,
        XtNfromHoriz, reflectAmbient,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( reflectAmbientScroll, XtNjumpProc, set_refl_props, (XtPointer)AMBIENT_COEF );

    sprintf( string, "%.2f", refl_props.ambient );
    reflectAmbientLabel = XtVaCreateManagedWidget( "AmbientLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectLabel,
        XtNfromHoriz, reflectAmbientScroll,
        XtNlabel, string,
        NULL );

    reflectDiffuse = XtVaCreateManagedWidget( "Diffuse",
        labelWidgetClass,
        reflectForm,
        XtNlabel, "Diff Coeff:",
        XtNwidth, 122,
        XtNfromVert, reflectAmbient,
        NULL );

    topOfThumb = refl_props.diffuse;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    reflectDiffuseScroll = XtVaCreateManagedWidget( "DiffuseScroll",
        scrollbarWidgetClass,
        reflectForm,
        XtNfromVert, reflectAmbient,
        XtNfromHoriz, reflectDiffuse,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( reflectDiffuseScroll, XtNjumpProc, set_refl_props, (XtPointer)DIFFUSE_COEF );

    sprintf( string, "%.2f", refl_props.diffuse );
    reflectDiffuseLabel = XtVaCreateManagedWidget( "DiffuseLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectAmbient,
        XtNfromHoriz, reflectDiffuseScroll,
        XtNlabel, string,
        NULL );

    reflectTransmission = XtVaCreateManagedWidget( "Transmission",
        labelWidgetClass,
        reflectForm,
        XtNlabel, "Transm Factor:",
        XtNwidth, 122,
        XtNfromVert, reflectDiffuse,
        NULL );

    topOfThumb = refl_props.transmission;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    reflectTransmissionScroll = XtVaCreateManagedWidget( "TransmissionScroll",
        scrollbarWidgetClass,
        reflectForm,
        XtNfromVert, reflectDiffuse,
        XtNfromHoriz, reflectTransmission,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( reflectTransmissionScroll, XtNjumpProc, set_refl_props, (XtPointer)TRANSMISSION_COEF );

    sprintf( string, "%.2f", refl_props.transmission );
    reflectTransmissionLabel = XtVaCreateManagedWidget( "TransmissionLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectDiffuse,
        XtNfromHoriz, reflectTransmissionScroll,
        XtNlabel, string,
        NULL );

    reflectSpecular = XtVaCreateManagedWidget( "Specular",
        labelWidgetClass,
        reflectForm,
        XtNlabel, "Spec Coeff:",
        XtNwidth, 122,
        XtNfromVert, reflectTransmission,
        NULL );

    topOfThumb = refl_props.specular;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    reflectSpecularScroll = XtVaCreateManagedWidget( "SpecularScroll",
        scrollbarWidgetClass,
        reflectForm,
        XtNfromVert, reflectTransmission,
        XtNfromHoriz, reflectSpecular,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( reflectSpecularScroll, XtNjumpProc, set_refl_props, (XtPointer)SPECULAR_COEF );

    sprintf( string, "%.2f", refl_props.specular );
    reflectSpecularLabel = XtVaCreateManagedWidget( "SpecularLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectTransmission,
        XtNfromHoriz, reflectSpecularScroll,
        XtNlabel, string,
        NULL );
    XtVaGetValues( reflectSpecularLabel, XtNwidth, &width, NULL );

    reflectSpecularConc = XtVaCreateManagedWidget( "SpecularConc",
        labelWidgetClass,
        reflectForm,
        XtNlabel, "Spec Exp:",
        XtNwidth, 122,
        XtNfromVert, reflectSpecular,
        NULL );

    topOfThumb = refl_props.specular_conc / 100.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    reflectSpecularConcScroll = XtVaCreateManagedWidget( "SpecularConcScroll",
        scrollbarWidgetClass,
        reflectForm,
        XtNfromVert, reflectSpecular,
        XtNfromHoriz, reflectSpecularConc,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtVaGetValues( reflectSpecularConcScroll, XtNwidth, &widthScr, NULL );
    XtAddCallback( reflectSpecularConcScroll, XtNjumpProc, set_refl_props, (XtPointer)SPECULAR_EXP );

    sprintf( string, "%.2f", refl_props.specular_conc );
    reflectSpecularConcLabel = XtVaCreateManagedWidget( "SpecularConcLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectSpecular,
        XtNfromHoriz, reflectSpecularConcScroll,
        XtNlabel, string,
        XtNwidth, width,
        NULL );

    reflectColor = XtVaCreateManagedWidget( "Color", commandWidgetClass,
        reflectForm,
        XtNfromVert, reflectSpecularConc,
        XtNlabel, "Spec Color:",
        XtNwidth, 122,
        NULL );
    XtAddCallback( reflectColor, XtNcallback, editSpecColor, NULL );

    sprintf( string, "R: %.2f G: %.2f B: %.2f", refl_props.specular_color.value.rgb.red, refl_props.specular_color.value.rgb.green, refl_props.specular_color.value.rgb.blue );
    reflectColorLabel = XtVaCreateManagedWidget( "ColorLabel", labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectSpecularConc,
        XtNfromHoriz, reflectColor,
        XtNlabel, string,
        XtNwidth, 8+width+widthScr,
        NULL );

    materialColor = XtVaCreateManagedWidget( "materialColor",
        commandWidgetClass,
        reflectForm,
        XtNfromVert, reflectColor,
        XtNlabel, "Material Color:",
        XtNvertDistance, 5,
        XtNwidth, 122,
        NULL );
    XtAddCallback( materialColor, XtNcallback, editMaterialColor, NULL );

    sprintf( string, "R: %.2f G: %.2f B: %.2f", colorMaterial.rgb.red, colorMaterial.rgb.green, colorMaterial.rgb.blue );
    materialColorLabel = XtVaCreateManagedWidget( "materialLabel",
        labelWidgetClass,
        reflectForm,
        XtNfromVert, reflectColor,
        XtNfromHoriz, materialColor,
        XtNlabel, string,
        XtNvertDistance, 5,
        XtNwidth, 8+width+widthScr,
        NULL );
#if( 0 ) 
    predefMaterialButton = XtVaCreateManagedWidget( "predefMaterialButton",
        menuButtonWidgetClass, reflectForm,
        XtNlabel, "Predefined Material",
        XtNfromVert, materialColor,
        XtNmenuName, "predefMaterialMenu",
        XtNvertDistance, 14,
        NULL );
    predefMaterialMenu = XtVaCreatePopupShell( "predefMaterialMenu",
        simpleMenuWidgetClass,
        predefMaterialButton,
        NULL );
#endif		/*  */
    quit = XtVaCreateManagedWidget( "quit", commandWidgetClass, form,
        XtNfromVert, canvas,
        NULL );
    XtAddCallback( quit, XtNcallback, finalQuit, NULL );
}

void
finalQuit( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    extern XtAppContext app;
    extern void printMaterial();

    printMaterial();
    XtDestroyApplicationContext( app );
    (void) exit (0);
}
